@@ -37,6 +37,14 @@ class DCA(IDCA):
     def on(self, val: bool):
         self.setter("on", 1 if val else 0)
 
+    @property
+    def mute(self) -> bool:
+        return not self.on
+
+    @mute.setter
+    def mute(self, val: bool):
+        self.on = not val
+
     @property
     def name(self) -> str:
         return self.getter("config/name")[0]
