@@ -1,18 +1,5 @@
 from dataclasses import dataclass
 
-"""
-# osc slightly different, interface would need adjusting to support this mixer.
-
-@dataclass
-class X32KindMap:
-    id_: str = "X32"
-    num_dca: int = 8
-    num_strip: int = 32
-    num_bus: int = 16
-    num_fx: int = 8
-    num_rtn: int = 6
-"""
-
 
 @dataclass
 class KindMap:
@@ -21,14 +8,24 @@ class KindMap:
 
 
 @dataclass
-class MR18KindMap(KindMap):
-    # note ch 17-18 defined as aux rtn
+class X32KindMap(KindMap):
+    id_: str
+    num_dca: int = 8
+    num_strip: int = 32
+    num_bus: int = 16
+    num_fx: int = 8
+    num_auxrtn: int = 8
+    num_matrix: int = 6
+
+
+@dataclass
+class XR18KindMap(KindMap):
+    # note ch 17-18 defined as aux return
     id_: str
     num_dca: int = 4
     num_strip: int = 16
     num_bus: int = 6
     num_fx: int = 4
-    num_rtn: int = 4
 
 
 @dataclass
@@ -38,7 +35,6 @@ class XR16KindMap(KindMap):
     num_strip: int = 16
     num_bus: int = 4
     num_fx: int = 4
-    num_rtn: int = 4
 
 
 @dataclass
@@ -48,12 +44,12 @@ class XR12KindMap(KindMap):
     num_strip: int = 12
     num_bus: int = 2
     num_fx: int = 4
-    num_rtn: int = 4
 
 
 _kinds = {
-    "XR18": MR18KindMap(id_="XR18"),
-    "MR18": MR18KindMap(id_="MR18"),
+    "X32": X32KindMap(id_="X32"),
+    "MR18": XR18KindMap(id_="MR18"),
+    "XR18": XR18KindMap(id_="XR18"),
     "XR16": XR16KindMap(id_="XR16"),
     "XR12": XR12KindMap(id_="XR12"),
 }
