@@ -1,8 +1,9 @@
 import abc
+import logging
 import threading
 import time
 from pathlib import Path
-from typing import Optional
+from typing import Optional, Union
 
 try:
     import tomllib
@@ -13,15 +14,15 @@ from pythonosc.dispatcher import Dispatcher
 from pythonosc.osc_message_builder import OscMessageBuilder
 from pythonosc.osc_server import BlockingOSCUDPServer
 
-from . import kinds
+from . import adapter, kinds
 from .bus import Bus
 from .config import Config
 from .dca import DCA
 from .errors import XAirRemoteError
-from .fx import FXReturn, FXSend
+from .fx import FX, FXSend
 from .kinds import KindMap
 from .lr import LR
-from .rtn import Aux, Rtn
+from .rtn import AuxRtn, FxRtn
 from .strip import Strip
 
 
@@ -30,15 +31,12 @@ class OSCClientServer(BlockingOSCUDPServer):
         super().__init__(("", 0), dispatcher)
         self.xr_address = address
 
-    def send_message(self, address: str, value: str):
+    def send_message(self, address: str, vals: Optional[Union[str, list]]):
         builder = OscMessageBuilder(address=address)
-        if value is None:
-            values = list()
-        elif isinstance(value, list):
-            values = value
-        else:
-            values = [value]
-        for val in values:
+        vals = vals if vals is not None else []
+        if not isinstance(vals, list):
+            vals = [vals]
+        for val in vals:
             builder.add_arg(val)
         msg = builder.build()
         self.socket.sendto(msg.dgram, self.xr_address)
@@ -47,21 +45,20 @@ class OSCClientServer(BlockingOSCUDPServer):
 class XAirRemote(abc.ABC):
     """Handles the communication with the mixer via the OSC protocol"""
 
+    logger = logging.getLogger("xair.xairremote")
+
     _CONNECT_TIMEOUT = 0.5
-    _WAIT_TIME = 0.025
-    _REFRESH_TIMEOUT = 5
 
-    XAIR_PORT = 10024
-
-    info_response = []
+    _info_response = []
 
     def __init__(self, **kwargs):
         dispatcher = Dispatcher()
         dispatcher.set_default_handler(self.msg_handler)
         self.xair_ip = kwargs["ip"] or self._ip_from_toml()
-        self.xair_port = kwargs["port"] or self.XAIR_PORT
-        if not (self.xair_ip and self.xair_port):
-            raise XAirRemoteError("No valid ip or password detected")
+        self.xair_port = kwargs["port"]
+        self._delay = kwargs["delay"]
+        if not self.xair_ip:
+            raise XAirRemoteError("No valid ip detected")
         self.server = OSCClientServer((self.xair_ip, self.xair_port), dispatcher)
 
     def __enter__(self):
@@ -79,26 +76,32 @@ class XAirRemote(abc.ABC):
     def validate_connection(self):
         self.send("/xinfo")
         time.sleep(self._CONNECT_TIMEOUT)
-        if len(self.info_response) > 0:
-            print(f"Successfully connected to {self.info_response[2]}.")
-        else:
-            print(
-                "Error: Failed to setup OSC connection to mixer. Please check for correct ip address."
+        if not self.info_response:
+            raise XAirRemoteError(
+                "Failed to setup OSC connection to mixer. Please check for correct ip address."
             )
+        print(
+            f"Successfully connected to {self.info_response[2]} at {self.info_response[0]}."
+        )
+
+    @property
+    def info_response(self):
+        return self._info_response
 
     def run_server(self):
         self.server.serve_forever()
 
     def msg_handler(self, addr, *data):
-        self.info_response = data[:]
+        self.logger.debug(f"received: {addr} {data if data else ''}")
+        self._info_response = data[:]
 
-    def send(self, address: str, param: Optional[str] = None):
-        self.server.send_message(address, param)
-        time.sleep(self._WAIT_TIME)
+    def send(self, addr: str, param: Optional[str] = None):
+        self.logger.debug(f"sending: {addr} {param if param is not None else ''}")
+        self.server.send_message(addr, param)
+        time.sleep(self._delay)
 
-    def _query(self, address):
+    def query(self, address):
         self.send(address)
-        time.sleep(self._WAIT_TIME)
         return self.info_response
 
     def __exit__(self, exc_type, exc_value, exc_tr):
@@ -109,11 +112,29 @@ def _make_remote(kind: KindMap) -> XAirRemote:
     """
     Creates a new XAIR remote class.
 
-    The returned class will subclass MAirRemote.
+    The returned class will subclass XAirRemote.
     """
 
-    def init(self, *args, **kwargs):
-        defaultkwargs = {"ip": None, "port": None}
+    def init_x32(self, *args, **kwargs):
+        defaultkwargs = {"ip": None, "port": 10023, "delay": 0.02}
+        kwargs = defaultkwargs | kwargs
+        XAirRemote.__init__(self, *args, **kwargs)
+        self.kind = kind
+        self.mainst = adapter.MainStereo.make(self)
+        self.mainmono = adapter.MainMono.make(self)
+        self.matrix = tuple(
+            adapter.Matrix.make(self, i) for i in range(kind.num_matrix)
+        )
+        self.strip = tuple(Strip.make(self, i) for i in range(kind.num_strip))
+        self.bus = tuple(adapter.Bus.make(self, i) for i in range(kind.num_bus))
+        self.dca = tuple(DCA(self, i) for i in range(kind.num_dca))
+        self.fx = tuple(FX(self, i) for i in range(kind.num_fx))
+        self.fxreturn = tuple(adapter.FxRtn.make(self, i) for i in range(kind.num_fx))
+        self.auxin = tuple(adapter.AuxRtn.make(self, i) for i in range(kind.num_auxrtn))
+        self.config = Config.make(self)
+
+    def init_xair(self, *args, **kwargs):
+        defaultkwargs = {"ip": None, "port": 10024, "delay": 0.02}
         kwargs = defaultkwargs | kwargs
         XAirRemote.__init__(self, *args, **kwargs)
         self.kind = kind
@@ -121,17 +142,25 @@ def _make_remote(kind: KindMap) -> XAirRemote:
         self.strip = tuple(Strip.make(self, i) for i in range(kind.num_strip))
         self.bus = tuple(Bus.make(self, i) for i in range(kind.num_bus))
         self.dca = tuple(DCA(self, i) for i in range(kind.num_dca))
+        self.fx = tuple(FX(self, i) for i in range(kind.num_fx))
         self.fxsend = tuple(FXSend.make(self, i) for i in range(kind.num_fx))
-        self.fxreturn = tuple(FXReturn(self, i) for i in range(kind.num_fx))
+        self.fxreturn = tuple(FxRtn.make(self, i) for i in range(kind.num_fx))
+        self.auxreturn = AuxRtn.make(self)
         self.config = Config.make(self)
-        self.aux = Aux.make(self)
-        self.rtn = tuple(Rtn.make(self, i) for i in range(kind.num_rtn))
 
+    if kind.id_ == "X32":
+        return type(
+            f"XAirRemote{kind}",
+            (XAirRemote,),
+            {
+                "__init__": init_x32,
+            },
+        )
     return type(
         f"XAirRemote{kind}",
         (XAirRemote,),
         {
-            "__init__": init,
+            "__init__": init_xair,
         },
     )
 
